// enforce semicolons after each code statement
#pragma semicolon 1

#include <sourcemod>
#include <sdktools>

#define PLUGIN_VERSION "2.0"

/*****************************************************************


			P L U G I N   I N F O


*****************************************************************/

public Plugin:myinfo = {
	name = "Voice Join",
	author = "Berni",
	description = "Allows players to join voiceservers ingame",
	version = PLUGIN_VERSION,
	url = "http://forums.alliedmods.net/showthread.php?p=674090"
}



/*****************************************************************


			G L O B A L   V A R S


*****************************************************************/

// ConVar Handles
new Handle:ts_version = INVALID_HANDLE;
new Handle:ts_printjoins = INVALID_HANDLE;

// Paths
new String:path_voiceservers[PLATFORM_MAX_PATH];
new String:path_users[PLATFORM_MAX_PATH];

// Keyvalue Handles
new Handle:kv_voiceservers = INVALID_HANDLE;



/*****************************************************************


			F O R W A R D   P U B L I C S


*****************************************************************/

public OnPluginStart() {
	
	// ConVars
	ts_version = CreateConVar("ts_version", PLUGIN_VERSION, "VoiceJoin plugin version", FCVAR_PLUGIN|FCVAR_DONTRECORD|FCVAR_REPLICATED|FCVAR_NOTIFY);
	// Set it to the correct version, in case the plugin gets updated...
	SetConVarString(ts_version, PLUGIN_VERSION);
	
	ts_printjoins = CreateConVar("ts_printjoins", "1", "Enable/disable voicejoin join messages <1/0>", FCVAR_PLUGIN);
	
	AutoExecConfig(true);
	
	BuildPath(Path_SM, path_voiceservers, sizeof(path_voiceservers), "configs/voicejoin/servers.cfg");
	BuildPath(Path_SM, path_users, sizeof(path_users), "configs/voicejoin/users.cfg");
	
	LoadTranslations("common.phrases");
	LoadTranslations("voicejoin.phrases");
	RegVoiceCommands();
}

public OnConfigsExecuted() {
	
	//RegVoiceCommands();
}



/****************************************************************


			C A L L B A C K   F U N C T I O N S


****************************************************************/

public Action:Command_VoiceJoin(client, args) {
	
	if (client == 0) {
		return Plugin_Handled;
	}
	
	decl String:command[64];
	GetCmdArg(0, command, sizeof(command));
	
	decl String:buffer[1024];
	decl String:serverType[64];
	new bool:found=false;
	if (KvGotoFirstSubKey(kv_voiceservers)) {

		do {

			KvGetSectionName(kv_voiceservers, serverType, sizeof(serverType));
			if (StrEqual(serverType, "")) {
				continue;
			}
			
			decl String:commandBuffer[64];

			KvGetString(kv_voiceservers, "command", buffer, sizeof(buffer));
			Format(commandBuffer, sizeof(commandBuffer), "sm_%s", buffer);
			if (StrEqual(buffer, command, false) || StrEqual(commandBuffer, command, false)) {
				found = true;
				break;
			}

			KvGetString(kv_voiceservers, "alias", buffer, sizeof(buffer));
			Format(commandBuffer, sizeof(commandBuffer), "sm_%s", buffer);
			if (StrEqual(buffer, command, false) || StrEqual(commandBuffer, command, false)) {
				found = true;
				break;
			}
		}
		while (KvGotoNextKey(kv_voiceservers));
	}
	
	if (!found) {
		ReplyToCommand(client, "[Voice Join] %t", "Error cmdnotfoundincfg");
		return Plugin_Handled;
	}
	
	decl String:type[64];
	KvGetSectionName(kv_voiceservers, type, sizeof(type));

	decl target_list[MAXPLAYERS];
	new target_count;

	// Normal usage
	if (args == 0) {

		target_count = 1;
		target_list[0] = client;
	}
	// Admin usage
	else if (args >= 1) {
	
		new AdminId:aid = GetUserAdmin(client);
		
		if (aid != INVALID_ADMIN_ID && GetAdminFlag(aid, Admin_Chat)) {
			decl String:target[MAX_TARGET_LENGTH];
			GetCmdArg(1, target, sizeof(target));

			decl String:target_name[MAX_TARGET_LENGTH];
			decl bool:tn_is_ml;

			target_count = ProcessTargetString(
					target,
					client,
					target_list,
					sizeof(target_list),
					COMMAND_FILTER_NO_BOTS,
					target_name,
					sizeof(target_name),
					tn_is_ml
			);
			
			ReplyToCommand(client, "\x04[Voice Join] %t", "Command executed", target);
		}
		else {
			ReplyToCommand(client, "\x04[Voice Join] %t", "Access denied");
			return Plugin_Handled;
		}
	}
	
	if (target_count <= 0) {
		ReplyToCommand(client, "\x04[Voice Join] %t", "No valid targets");
		return Plugin_Handled;
	}
	
	new String:url[1024] = "\0";
	new String:host[256] = "\0";
	new String:port[8] = "\0";
	new String:serverPassword[64] = "\0";
	new String:disAllowedClientNameChars[128] = "\0";
	new String:channel[128] = "\0";
	new String:channelPassword[64] = "\0";
	new String:nickName[MAX_NAME_LENGTH] = "\0";
	new String:login[64] = "\0";
	new String:password[64] = "\0";
	new String:userPassword[64] = "\0";
	
	KvGetString(kv_voiceservers, "url", url, sizeof(url));
	KvGetString(kv_voiceservers, "host", host, sizeof(host));
	KvGetString(kv_voiceservers, "port", port, sizeof(port));
	KvGetString(kv_voiceservers, "serverpassword", serverPassword, sizeof(serverPassword));
	strcopy(password, sizeof(password), serverPassword);
	KvGetString(kv_voiceservers, "disallowedclientnamechars", disAllowedClientNameChars, sizeof(disAllowedClientNameChars));

	new Handle:kv_users = CreateKeyValues("users");
	
	if (!FileToKeyValues(kv_users, path_users)) {
		kv_users = INVALID_HANDLE;
		CloseHandle(kv_users);
	}
	
	for (new i=0; i<target_count; i++) {
		
		new player = target_list[i];
		
		if (IsClientAuthorized(player)) {
		
			// Let's parse users.cfg first
			decl String:auth[32];
			GetClientAuthString(player, auth, sizeof(auth));
			
			if (kv_users != INVALID_HANDLE && KvJumpToKey(kv_users, auth)) {
					
				if (KvJumpToKey(kv_users, type)) {
					
					KvGetString(kv_users, "login", buffer, sizeof(buffer));
					if (!StrEqual(buffer, "")) {
						strcopy(login, sizeof(login), buffer);
						KvGetString(kv_users, "password", userPassword, sizeof(userPassword));
						strcopy(password, sizeof(password), userPassword);
					}

					KvGetString(kv_users, "nickname", buffer, sizeof(buffer));
					if (!StrEqual(buffer, "")) {
						strcopy(nickName, sizeof(nickName), buffer);
					}

					KvGetString(kv_users, "channel", buffer, sizeof(buffer));
					if (!StrEqual(buffer, "")) {
						strcopy(channel, sizeof(channel), buffer);
						KvGetString(kv_users, "channelpassword", channelPassword, sizeof(channelPassword));
					}
				}
				
				KvRewind(kv_users);
			}
		}
		
		if (StrEqual(channel, "")) {
			
			if (KvJumpToKey(kv_voiceservers, "channels")) {
				
				if (KvJumpToKey(kv_voiceservers, "teams")) {
					
					decl String:player_team_str[64];
					new player_team = GetClientTeam(player);
					GetTeamName(player_team, player_team_str, sizeof(player_team_str));

					if (KvGotoFirstSubKey(kv_voiceservers)) {

						do {
							
							KvGetSectionName(kv_voiceservers, buffer, sizeof(buffer));
							
							if (StringToInt(buffer) != player_team && StrContains(player_team_str, buffer, false) == -1) {
								continue;
							}

							KvGetString(kv_voiceservers, "channel", channel, sizeof(channel));
							KvGetString(kv_voiceservers, "channelpassword", channelPassword, sizeof(channelPassword), "");
							break;
						
						}
						while (KvGotoNextKey(kv_voiceservers));

						KvGoBack(kv_voiceservers);
					}
					
					KvGoBack(kv_voiceservers);
				}
			}
		}

		if (StrEqual(channel, "")) {
			
			KvJumpToKey(kv_voiceservers, "default");
			
			KvGetString(kv_voiceservers, "channel", channel, sizeof(channel));
			KvGetString(kv_voiceservers, "channelpassword", channelPassword, sizeof(channelPassword));
		}
		
		if (StrEqual(nickName, "")) {
			GenerateValidNickName(player, nickName, sizeof(nickName), disAllowedClientNameChars);
		}

		UrlDataEncode(host,				sizeof(host));
		UrlDataEncode(nickName,			sizeof(nickName));
		UrlDataEncode(login,			sizeof(login));
		UrlDataEncode(password,			sizeof(password));
		UrlDataEncode(serverPassword,	sizeof(serverPassword));
		UrlDataEncode(userPassword,		sizeof(userPassword));
		UrlDataEncode(channel,			sizeof(channel));
		UrlDataEncode(channelPassword,	sizeof(channelPassword));
		
		ReplaceString(url, sizeof(url), "$host",			host);
		ReplaceString(url, sizeof(url), "$port",			port);
		ReplaceString(url, sizeof(url), "$nickname",		nickName);
		ReplaceString(url, sizeof(url), "$login",			login);
		ReplaceString(url, sizeof(url), "$password",		password);
		ReplaceString(url, sizeof(url), "$serverpassword",	serverPassword);
		ReplaceString(url, sizeof(url), "$userpassword",	userPassword);
		ReplaceString(url, sizeof(url), "$channel",			channel);
		ReplaceString(url, sizeof(url), "$channelpassword",	channelPassword);

		ReplyToCommand(player, "\x04[Voice Join] %t", "Joining server", serverType);
		ShowMOTDPanel(player, "", url, MOTDPANEL_TYPE_URL);
		ShowVGUIPanel(player, "info", INVALID_HANDLE, false);
		
		if (args == 0 && GetConVarBool(ts_printjoins)) {
			decl String:playerName[MAX_NAME_LENGTH];
			GetClientName(player, playerName, sizeof(playerName));
			PrintToChatAll("\x04[Voice Join] %T", "Player is joining", player, playerName, serverType);
		}
	}
	
	if (kv_users != INVALID_HANDLE) {
		CloseHandle(kv_users);
	}
	KvRewind(kv_voiceservers);
	
	return Plugin_Handled;
}

/*****************************************************************


			P L U G I N   F U N C T I O N S


*****************************************************************/

RegVoiceCommands() {
	
	if (kv_voiceservers != INVALID_HANDLE) {
		CloseHandle(kv_voiceservers);
	}
	kv_voiceservers = CreateKeyValues("voiceservers");

	if (!FileToKeyValues(kv_voiceservers, path_voiceservers) || !KvGotoFirstSubKey(kv_voiceservers)) {
		CloseHandle(kv_voiceservers);
		kv_voiceservers = INVALID_HANDLE;
		return;
	}
 
	decl String:buffer[64];
	do {

		KvGetSectionName(kv_voiceservers, buffer, sizeof(buffer));
		if (StrEqual(buffer, "")) {
			continue;
		}
		
		KvGetString(kv_voiceservers, "enabled", buffer, sizeof(buffer), "0");
		if (!StringToInt(buffer)) {
			continue;
		}

		KvGetString(kv_voiceservers, "command", buffer, sizeof(buffer));
		if (StrEqual(buffer, "")) {
			continue;
		}
		RegVoiceJoinCmd(buffer);

		KvGetString(kv_voiceservers, "alias", buffer, sizeof(buffer));
		if (!StrEqual(buffer, "")) {
			RegVoiceJoinCmd(buffer);
		}
	}
	while (KvGotoNextKey(kv_voiceservers));

	KvRewind(kv_voiceservers);
}

GenerateValidNickName(client, String:buffer[], size, const String:disallowedChars[] = "") {

	decl String:name[MAX_NAME_LENGTH];
	GetClientName(client, name, sizeof(name));
	
	new name_i=0, buffer_i=0;
	while (name[name_i] != '\0' && (buffer_i-1) < size) {
		
		if (FindCharInString(disallowedChars, name[name_i]) == -1) {
			buffer[buffer_i] = name[name_i];
			buffer_i++;
		}
		name_i++;
	}
	
	buffer[buffer_i] = '\0';
}

UrlDataEncode(String:str[], size) {
	
	ReplaceString(str, size, "%", "%25");
	ReplaceString(str, size, " ", "%20");
	ReplaceString(str, size, "!", "%21");
	ReplaceString(str, size, "\"", "%22");
	ReplaceString(str, size, "#", "%23");
	ReplaceString(str, size, "$", "%24");
	ReplaceString(str, size, "&", "%26");
	ReplaceString(str, size, "'", "%27");
	ReplaceString(str, size, "(", "%28");
	ReplaceString(str, size, ")", "%29");
	ReplaceString(str, size, "@", "%40");
	ReplaceString(str, size, "`", "%60");
	ReplaceString(str, size, "", "%91");
	ReplaceString(str, size, "", "%92");
}

RegVoiceJoinCmd(const String:cmd[]) {
	
	decl String:voiceJoinCommand[64];
	
	if (StrContains(cmd, "sm_") == 0) {
		Format(voiceJoinCommand, sizeof(voiceJoinCommand), "%s", cmd);
	}
	else {
		Format(voiceJoinCommand, sizeof(voiceJoinCommand), "sm_%s", cmd);
	}

	RegConsoleCmd(voiceJoinCommand, Command_VoiceJoin, "[Voice Join] Allows players to connect to the voiceserver ingame", FCVAR_PLUGIN);
}
